
from code_monger import CodeGenerator

class EmailCodeVerifier:
	'''
	This class is used to verify code sent to emails 

	The codes are saved in a file .

	Attributes
	----------
	name:str
		A name you want to call the code verifier, any can do .
	file:str
		A path to a file where you wish to store the codes.
	length:int , optional
		The length of the codes to be generated.
	letters:bool , optional 
		Set to `True` to include alphabet letters  	.	
	punctuation_chars:bool
		Set to `True` to include punctuation charachters.
	casing:bool
		Set to `upper` , `lower` or `all` to set the casing of the code .
		Options are:
			upper - uppercase letters only
			lower - lowercase letters only
			all   - include all cases 

	Methods
	-------
	ValidateCode ( code:str , email:str , delete_if_valid:bool )-> bool
		This method checks if a code is valid ( present in the codes file ),
		along with the email provided . 

		Returns
		--------
		bool
			Whether the code provided was valid or not
	MakeNewCode( email:str) -> str
		Parameters
		----------
		email:str
			The email address asociated with the code to be generated.

		Returns
		-------
		str
			The code generated
	ClearCodes() -> None
		This method clears all the data in the file where the codes are stored.
	
	'''

	def __init__(self ,
						name:str ,
						  file:str , 
						length:int = 6,
						 numbers:bool = True ,
						  letters:bool = False , 
						punctuation_chars:bool = False ,
						 casing:str = "upper" ):
		'''
		Parameters
		----------
		name:str
			A name you want to call the code verifier, any can do .
		file:str
			A path to a file where you wish to store the codes.
		length:int , optional
			The length of the codes to be generated.
		letters:bool , optional 
			Set to `True` to include alphabet letters . 		
		punctuation_chars:bool
			Set to `True` to include punctuation charachters.
		casing:bool
			Set to `upper` , `lower` or `all` to set the casing of the code .
			Options are:
				upper - uppercase letters only
				lower - lowercase letters only
				all   - include all cases 
		'''

		self.file = file
		self.code_generator =CodeGenerator(storage_file = file )
		self.length = length 
		self.numbers = numbers 
		self.letters =  letters 
		self.punctuation_chars = punctuation_chars
		self.casing = casing
		self.name = name

	def ValidateCode(self , code , email , delete_if_valid = True ) -> bool:
		'''
		Used to check whether the code and email provided are present in the file. ie. If they are valid

		Parameters
		----------
		code:str
			A string representing the code that is to be validated .
		email:str
			The email address that was used during code generation .
		delete_if_valid:bool
			Set to `True` to delete the code and email from the file if the code is valid .
		
		Returns
		-------
		bool
			True if the code is valid else False

		'''
		
		return self.code_generator.ValidateCode(code , email , delete_if_valid)

	def MakeNewCode(self , email:str ) ->str:
		'''This method generates and saves a new code in the storage file provided  

		Parameters
		----------
		email:str
			The email address asociated with the code to be generated.

		Returns
		-------
		str
			The code generated
		'''

		code = self.code_generator.NewCode(
				email , length = self.length ,
				 numbers = self.numbers , letters = self.letters ,
				  punctuation_chars = self.punctuation_chars ,
				  casing = self.casing,

				   )
		return code

	def ClearCodes(self) -> None:
		'''
		Deletes all codes from the storage file 
		'''

		print("Removing all codes ...")
		with open(self.file , "w" , encoding='utf-8') as f:
			f.write('')
		print("All codes have been removed")




import os

from .email_template import EmailTemplate 
from .email_sender import GmailSender , EmailSender
from .email_code_verifier import EmailCodeVerifier

class EmailWorker:
	"""
	This is a base class that can send emails and validates email codes.

	storage_folder : where the auth codes files and other files will be stored
	email ,password : Authentication information

	Attributes
	-----------
	email:str
		The email address used to send the email.
	password:str
		The password to the provided email account.
	storage_folder:str , optional
		A directory where files will be stored.

	Methods
	--------
	VerifyCode( code_verifier:EmailCodeVerifier , code:str , email:str , delete_if_valid = False) 
		Parameters
		-----------
		code_verifier:EmailCodeVerifier
			An EmailCodeVerifier object for code verification.
		code:str
			The code to be verified.
		delete_if_valid:bool , optional
			Set to `True` to delete the code if it is valid and vice versa.

		Returns
		-------
		bool
			Whether the code is valid or not
	 MakeNewCode(  code_verifier:EmailCodeVerifier , email:str )
	 	Generate a new code with the email as an identifier in the file .

		Parameters
		-----------
		code_verifier:EmailCodeVerifier
			An EmailCodeVerifier object for code generation.
		email:str
			An email address to be associated with the code .

		Returns
		-------
		str
			The code generated

	"""

	def __init__(self  , email:str , password:str ,storage_folder:str= "./"   ):
		self.storage_folder = storage_folder 
		if not os.path.isdir(storage_folder):
			os.makedirs(storage_folder)
		self.email = email 
		self.password = password 

	def VerifyCode(self , code_verifier:str , code:str , email:str , delete_if_valid = False) -> bool:
		'''Verifies a code that was generated and stored in the storage file

		Parameters
		-----------
		code_verifier:EmailCodeVerifier
			An EmailCodeVerifier object for code verification.
		code:str
			The code to be verified.
		delete_if_valid:bool , optional
			Set to `True` to delete the code if it is valid and vice versa.

		Returns
		-------
		bool
			Whether the code is valid or not
		'''
		# check if the code verifier with the name provided exists 
		return code_verifier.ValidateCode(code ,email , delete_if_valid = delete_if_valid)

	def MakeNewCode(self , code_verifier:EmailCodeVerifier , email:str ):
		'''Generate a new code with the email as an identifier in the file 

		Parameters
		-----------
		code_verifier:EmailCodeVerifier
			An EmailCodeVerifier object for code generation.
		email:str
			An email address associated to be with the code .

		Returns
		-------
		str
			The code generated
		'''

		code = code_verifier.MakeNewCode(email)
		return code 
		 



class GmailEmailWorker(EmailWorker):
	'''
	An EmailWorker child class for sending gmail emails .

	Attributes
	---------
	email:str
		The email address used to send the email
	password:str
		The gmail app password to the provided gsmail account
	storage_folder:str , optional
		A directory where files will be stored

	Methods
	--------
	SendTemplate(  template:EmailTemplate , subject:str  ,    recipient:str ) -> bool
		Parameters
		-----------
		template:EmailTemplate
			The EmailTemplate object you wish to send
		subject:str 
			The subject of the email
		recipient:str
			An email address that will receive the email
		
		Returns
		-------
		bool
			Whether or not the email was sent

	SendTextEmail(  message:str , recipient:list , subject:str) -> bool
		Parameters
		-----------
		message:str
			The message in text form to ve sent
		subject:str 
			The subject of the email
		recipient:str
			An email address that will receive the email
		
		Returns
		-------
		bool
			Whether or not the email was sent
	'''


	def __init__(self ,  email:str , password:str , storage_folder:str = "./" ):
		'''
		Parameters
		---------
		email:str
			The email address used to send the email
		password:str
			The password to the provided email account
		storage_folder:str , optional
			A directory where files will be stored

		'''
		EmailWorker.__init__(self ,   email , password   , storage_folder = storage_folder )

	def SendTemplate(self , template:EmailTemplate , subject:str  ,    recipient:str ) -> bool:
		'''
		Send an email to the provided recipients

		Parameters
		-----------
		template:EmailTemplate
			The EmailTemplate object you wish to send
		subject:str 
			The subject of the email
		recipient:str
			An email address that will receive the email
		
		Returns
		-------
		bool
			Whether or not the email was sent

		'''
		print("Sending  [{}] email to {} ".format(template.name , recipient))
		print("Sending email -----", end = "")
		sender = GmailSender(self.email , self.password)
		content = template.content
		plain_text_content = template.plain_text_content
		send_result =  sender.SendHTMLEmail(recipient , subject , content , plain_text_content)
		print("SENT" if send_result else "FAILED")
		return sendmail

	def SendTextEmail(self , message:str , recipient:list , subject:str) -> bool:
		'''Sends a text based email to the recipient

		Parameters
		-----------
		message:str
			The message in text form to ve sent
		subject:str 
			The subject of the email
		recipient:str
			An email address that will receive the email
		
		Returns
		-------
		bool
			Whether or not the email was sent

		'''
		print("Sending  [{}] email to {} ".format("Text based email" , recipient))
		print("Sending email ..." , end='')
		sender = GmailSender(self.email , self.password)
		send_result = sender.SendTextEmail(recipient , subject , message )
		print("SENT" if send_result else "FAILED")
		return sendmail


 





class  EmailTemplateLoadingError(Exception):
	'''
	An exception to be raised when a problem occurs during template loading
	'''
	def __init__(self , message):
		self.message = message
		super().__init__(self.message)



 







import os
from os.path import join 

current_file = __file__
base_dir ,file_name = os.path.split(current_file)
boilerplates_dir = join(base_dir , "html_boilerplates")

email_verification_templates_base_dir = join(boilerplates_dir , "email_verification")
verification_code_templates_base_dir = join(boilerplates_dir , "verification_code")


EMAIL_VERIFICATION_TEMPLATES = {
		1:join( email_verification_templates_base_dir , "1.html" ),
		2:join( email_verification_templates_base_dir , "2.html" ),
		3:join( email_verification_templates_base_dir , "3.html" ),

}

VERIFICATION_CODE_TEMPLATES = {
		1:join( verification_code_templates_base_dir, "1.html" ),
		2:join( verification_code_templates_base_dir, "2.html" ),
		3:join( verification_code_templates_base_dir, "3.html" ),

	
}










import smtplib, ssl
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.message import EmailMessage




class EmailSender:
	'''
	a base class that facilitates sending of emails 

	Attributes
	-----------
	email:str
		Email address to use when sending emails
	password:str
		Password to the email
	'''

	def __init__(self , email , password):
		'''
		Parameters
		----------
		email:str
			Email address to use when sending emails
		password:str
			Password to the email
		'''

		self.email = email 
		self.password = password

 
	 

class GmailSender(EmailSender):
	'''
	This class is for sending gmail emails 

	Attributes
	-----------
	email:str
		Email address to use when sending emails
	password:str
		The Gmail app Password to the gmail account

	Methods
	-------
	SendHTMLEmail(email:str , password:str)
		Sends an html email to the provided recipient
		
		Parameters
		----------
		recipients:str
			email address to receive the email .
		subject:str
			The subject of the email to be sent .
		html:
			The html code to send 
		plain_message:str
			Alternative message in text form (To be shown if the HTML doesn't render)


		Returns
		-------
		bool
			True if the email is sent successfully ,else False

	'''

	def __init__(self , email:str , password:str):
		'''
		email:str
			Email address to use when sending emails
		password:str
			The Gmail app Password to the gmail account
		'''
		EmailSender.__init__(self , email , password)


	def SendHTMLEmail(self ,recipient:str, subject:str , html:str, plain_message:str = "" ) -> bool:
		'''
		Sends an html email to the provided recipient
		
		Parameters
		----------
		recipients:str
			email address to receive the email .
		subject:str
			The subject of the email to be sent .
		html:
			The html code to send 
		plain_message:str
			Alternative message in text form (To be shown if the HTML doesn't render)


		Returns
		-------
		bool
			True if the email is sent successfully ,else False

		'''
		print("Preparing to send HTML message to {} from sender {}. [PASSWORD = {} ]".format(recipient, self.email , self.password[:3] + ("*"*(len(self.password)-3))))
		port = 465  # For SSL
		# Create a secure SSL context
		# strip the tags from the html if the plain_message is not present
		message = MIMEMultipart("alternative")
		message["Subject"] = subject
		message["From"] = self.email
		message["To"] = recipient
		# load the html and plain messages as  MIMEText objects
		part1 = MIMEText(plain_message, "plain")
		part2 = MIMEText(html, "html")
		# Add HTML/plain-text parts to MIMEMultipart message
		# The email client will try to render the last part first
		message.attach(part1)
		message.attach(part2)
		context = ssl.create_default_context()
		with smtplib.SMTP_SSL("smtp.gmail.com", port, context=context) as server:
			try:
				server.login(self.email ,self.password)
				server.sendmail(self.email , recipient ,message.as_string())
				return True
			except Exception as e:
				raise
		return False


	def SendTextEmail(self , recipient:str, subject:str , message:str ) -> bool:
		'''
		Send an text based email to the provided recipients

		Parameters
		----------
		recipient:str
			The email address to receive the email .
		subject:str
			The subject of the email to be sent .
		message:
			The message to send 

		Returns
		-------
		bool
			Whether the email has been sent or not

		'''

		msg = EmailMessage()
		msg['Subject'] = subject
		msg['From'] =self.email
		msg['To'] = recipient
		msg.set_content(message)
		print("Preparing to send a text based message to {} from sender {}. [PASSWORD = {} ]".format(recipient, self.email , self.password[:3] + ("*"*(len(self.password)-3))))
		port = 465  # For SSL
		# Create a secure SSL context
		context = ssl.create_default_context()
		with smtplib.SMTP_SSL("smtp.gmail.com", port, context=context) as server:
			try:
				server.login(self.email ,self.password)
				server.send_message(msg)
				return True
			except Exception as e:
				raise
		return False




'''

Before running this test , make sure a file called [my_email_auth] exists in the root directory 
in the file incluse the following variables

		- GMAIL_EMAIL - An email address you own that will send the email
		- GMAIL_APP_PASSWORD - Gmail app password (NOT THE NORMAL PASSWORD)
		- TEST_RECEIVER_EMAIL - A different  email address you own that will receive the test emauk

  '''


import unittest
from unittest import TestCase


from wholemail import EmailWorker ,GmailEmailWorker , EmailTemplate , GmailSender
from my_email_auth import GMAIL_EMAIL , GMAIL_APP_PASSWORD , TEST_RECIPIENT

from tests.data_for_tests import *


test_sender = GmailSender(GMAIL_EMAIL , GMAIL_APP_PASSWORD )

class Test_GmailSending(TestCase):
	'''
	Test functionalities of the GmailEmailWOrker class. ie test sending emails via gmail
	'''
	def test_SendHTMLEmail(self):
		'''
		test sending a html email using gmail
		'''
		sender = test_sender
		html  = test_email_template.content
		plain_message = test_email_template.plain_text_content
		subject = "Testing wholemail gmail mail sending"
		result = sender.SendHTMLEmail( TEST_RECIPIENT , subject , html , plain_message  )
		self.assertTrue(result)


	def test_SendTextEmail(self):
		'''
		test sending a text based email using gmail
		'''

		sender = test_sender
		message = "Just testing"
		subject = "Testing wholemail gmail mail sending"
		result = sender.SendTextEmail( TEST_RECIPIENT , subject ,message  )
		self.assertTrue(result)




import os

from .email_template import EmailTemplate , FileEmailTemplate
from .exceptions import EmailTemplateLoadingError




def LoadTemplate( html_text = "" , html_file = ""  , context ={}  , name = EmailTemplate.name ):
	'''
	This function creates an EmailTemlate opbject based on the information provided

	NOTE : 	Provide only either html_text or html_file parameters , 
			if both are provided , the html_file will be used, if it has issues, the html text will be used,
			if both html_file and html_text have issues, an exception will be raised
	
	Parameters
	----------
	html_text:str
		HTML content you want to send , it should contain the {{value}} blocks for context rendering
	html_file:str , optional
		The path of a html file you wish to create a template from
	context:dict , optional
		A dict containing the values you want inserted into the html content . 
	name:str , optional
		The name you want to give to the template

	Returns
	-------
	EmailTemplate


	'''

	# chec the html file
	if not os.path.isfile(html_file):
		try:
			folder , file = os.path.split(html_file)
			if not os.path.isdir(folder):
				# make the folder since its not there
				os.makedirs(folder)
				# make the html file
			with open(html_file , "w" , encoding ='utf-8') as f:
				f.write("")

		except FileNotFoundError:
			raise EmailTemplateLoadingError("The file could not be found or created")
		except Exception as e:
			raise EmailTemplateLoadingError("Could not load the template html content because of {}".format(e))
	
	html_content = ""
	html_file_content = ""
	with open(html_file , "r" , encoding ="utf-8") as f:
		html_file_content = f.read()
	# select which content to use
	if html_file_content and html_text:
		html_content = html_file_content
	elif html_file_content and not html_text:
		html_content = html_file_content
	elif html_text and not html_file_content:
		html_content = html_text
	elif not html_file_content and not html_text:
		html_content = ""
	template = EmailTemplate(html_content , context =context , name = name  )
	return template


'''

Here is what is happening:
	# test update content in EmailTemplate
	# test edit context in EmailTemplate
	# test FileEmailTemplate file loading 


'''
import unittest ,os
from unittest import TestCase

from wholemail import EmailTemplate , FileEmailTemplate

test_html_text = """
{{name}} ( born Albert Chinụalụmọgụ Achebe; {{DOB}} – 21 March 2013)
 was a Nigerian novelist, poet, and critic who is regarded as a central figure of modern African literature.
  His first novel and magnum opus, Things Fall Apart (1958), occupies a pivotal place in African literature and
   remains the most widely studied, translated, and read African novel.

    Along with Things Fall Apart, his No Longer at Ease (1960) and Arrow of God (1964) complete the "African Trilogy".
     Later novels include A Man of the People (1966) and Anthills of the Savannah (1987).
      Achebe is often referred to as the "father of modern African literature",
      although he vigorously rejected the characterization.

     """
test_context = {
	"name":"Chinua Achebe",
	"DOB":"16 November 1930",
}
test_html_file = "test_data/html/chinua_achebe.html"
test_email_template = EmailTemplate(test_html_text,test_context)

# some helper functions
def delete_test_html_file():
	'''
	Delete the html test file . 
	to be used during testing 
	'''
	if os.path.isfile(test_html_file ):
		os.remove(test_html_file)

def restore_test_html_file():
	'''
	Restore the html test file and write the test html text in it  
	to be used during testing 
	'''
	with open(test_html_file , "w" , encoding  ="utf-8") as f:
		f.write(test_html_text)



class TestEmailTemplate(TestCase):

	"""
	test the base class wholemail.EmailTemplate 

	"""

	def test_UpdatingContent(self):
		template = test_email_template
		# the UpdateContent function has alreadybeen run , let's test its results
		# test if the content attriute has changed and includes data from context
		print(template.content)
		self.assertTrue(test_context["name"] in template.content)
		# check if the html attr is intact
		self.assertTrue(template.html == test_html_text)

	def test_EditingContent(self):
		template = test_email_template
		# change the name in the content and see if all is Ok
		new_name = "Groot"
		template.Edit("name" ,new_name)
		self.assertTrue(new_name in template.content) 
		self.assertTrue(template.context["name"] == new_name)


class TestFileEmailTemplate(TestCase):

	"""
	Test the wholemail.FileEmailTemplate class 

	"""
	def test_FileLoading(self):
		# delete the file and see if it is re-created and works alright
		restore_test_html_file()
		template = FileEmailTemplate(test_html_file , test_context)
		# check if the context and content attrs are in order
		self.assertTrue(test_context["name"] in template.content)
		# check if the html attr is intact
		self.assertTrue(template.html == test_html_text)
		# de;ete the file and check stuff
		delete_test_html_file()
		with self.assertRaises(	FileNotFoundError):
			template = FileEmailTemplate(test_html_file , test_context)



import os 
import jinja2
from bs4 import BeautifulSoup

from . import html_boilerplates

TEMPLATE_STYLES_AVAILABLE= [1,2]

def strip_html_tags(html:str)-> str:
	'''
	Removes the tags from html content and return the plain text in between the tags .
	'''
	soup = BeautifulSoup(html , "html.parser")
	text  = soup.get_text()
	return text



class EmailTemplate:
	"""
	This class represents the HTML email you wish to send 
	"""

	name = "Template"
	def __init__(self , html, context = {} , name = "Template"):
		'''
		Parameters
		----------
		html:str
			The html code to send in the email 
		context:dict 
			Context information to be inserted into the html code eg. { 'name' : 'John' }
		name:str
			The name you wish to give to the template

		Methods
		-------
		Edit(key:str , value:str)
			Edit a variable in the context and update the html_content

			Parameters
			----------
			key:str
				The name of the value to be updated in the html code
			value:str
				The value you wish to insert into the html code
		UpdateContent()
			Update the data in the content attribute 
			Use this when editing/updating the context 
		'''
		self.name = name 
		self.context = context		 
		self.html = html # will not change ever
		self.content = ""
		self.plain_text_content = ""
		self.jinja_environment = jinja2.Environment()
		self.UpdateContent()

	def UpdateContent(self) -> None:
		'''
		Update the data in the content attribute 
		Use this when editing/updating the context 
		'''
		template = self.jinja_environment.from_string(self.html)
		content =  template.render(**self.context)
		self.content =content
		self.plain_text_content = strip_html_tags(content)

	def Edit(self, key:str , value:str):
		'''
		Edit a variable in the context and update the html_content

		Parameters
		----------
		key:str
			The name of the value to be updated in the html code
		value:str
			The value you wish to insert into the html code
	
		'''
		if not key in self.context:
			raise KeyError("The key provided is not in the context : {}".format(self.context))
		self.context[key] = value
		self.UpdateContent()

 

class FileEmailTemplate(EmailTemplate) :
	'''
	An EmailTemplate class where html content is loaded from a file 

	Parameters
	----------
	file:str
		The path to the file with the html code  
	context:dict 
		Context information to be inserted into the html code eg. { 'name' : 'John' }
	name:str
		The name you wish to give to the template
	'''
	def __init__(self ,file:str , context:str , name = "file-email-template" ):
		self.context = context
		self.file  =file
		if not os.path.isfile(file):
			raise FileNotFoundError("Please provide the file path ")
		with open(file , "r", encoding  ="utf-8") as f:
			html = f.read()

		EmailTemplate.__init__(self , html , context , name = name)

 


class BasicCodeVerificationEmailTemplate(FileEmailTemplate):
	'''
	A pre-set template for sending authentication codes to a users email 
	'''

	def __init__(self ,
		code ,
		recipient_name,
		company_name = "", 
		company_address = "",
		customer_support_link = "" ,
		company_phone_number = "",
		company_website_link = "",
		template_style = 1,
		name = 'code-verification',

					 ):
		# get the html template
		html_file = html_boilerplates.VERIFICATION_CODE_TEMPLATES[template_style]
		context = locals()
		context.pop("self")
		super().__init__(html_file , context , name = name)

class BasicEmailVerificationEmailTemplate(FileEmailTemplate):
	'''
	A pre-set template for sending authentication links to a users email 
	'''

	def __init__(self ,
					recipient_name , 
					company_name, 
					email_verification_link , 
					company_address = "",
					customer_support_link = "" ,
					company_phone_number = "",
					company_website_link = "",
					template_style = 1,
					name = "Simple-email-verification-template",
					 ):
		# get the html template
		html_file = html_boilerplates.EMAIL_VERIFICATION_TEMPLATES[template_style]
		context = locals()
		context.pop("self")
		super().__init__(html_file ,context , name = name )



 


import unittest
from unittest import TestCase

from tests.data_for_tests import *

from wholemail import EmailWorker ,GmailEmailWorker , EmailTemplate 


test_templates = {
	"auth":EmailTemplate( "<h1>Authentication</h1>"),
	"hello":EmailTemplate( "<h1>Hello there</h1>"),

}
test_email_worker = EmailWorker("email" ,"password" , storage_folder = "test_data" )
test_email = "testemail@dom"
test_code_verifier = test_code_verifiers['auth']

class Test_EmailWorker(TestCase):


	def test_CodeValidation(self):
		'''
		test if code validation is done correctly 
		'''

		worker = test_email_worker
		verifier = test_code_verifier
		verifier.ClearCodes()
		code = worker.MakeNewCode(verifier , test_email )
		self.assertTrue(len(code) == verifier.length )
		validation = worker.VerifyCode( verifier , code, test_email)
		self.assertTrue(validation)
		




from .email_template import EmailTemplate , FileEmailTemplate , TEMPLATE_STYLES_AVAILABLE, BasicCodeVerificationEmailTemplate, BasicEmailVerificationEmailTemplate
from .email_worker import EmailWorker , GmailEmailWorker
from .email_sender import EmailSender , GmailSender
from .email_code_verifier import EmailCodeVerifier
from .template_loader import LoadTemplate
from .exceptions import EmailTemplateLoadingError

__all__ = [
	"EmailTemplateLoadingError",
	"EmailCodeVerifier",
	"EmailTemplate", 
	"FileEmailTemplate",
	'GmailSender',
	'GmailEmailWorker',
	"LoadTemplate",
	'BasicCodeVerificationEmailTemplate',
	'BasicEmailVerificationEmailTemplate',

	]


 

import setuptools
from setuptools import setup, find_packages

long_description = ""
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="wholemail",
    version="0.1.5",
    author="Victor Kipkemboi",
    author_email="scriptilapia@gmail.com",
    long_description =long_description,
    long_description_content_type="text/markdown",
    description="A Python package built to facilitate easy sending of emails and other common generic functionality like sending authentication code emails.",
    url="https://github.com/victhepythonista/wholemail",
    project_urls={
        "Bug Tracker": "https://github.com/victhepythonista/wholemail/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License"
    ],
    packages=["wholemail"],
    python_requires=">=3.6",
    install_requires=[
          'Jinja2', "code_monger" , "bs4",
      ],
 
)



'''

Before running this test , make sure a file called [my_email_auth] exists in the root directory 
in the file incluse the following variables

		- GMAIL_EMAIL - An email address you own that will send the email
		- GMAIL_APP_PASSWORD - Gmail app password (NOT THE NORMAL PASSWORD)
		- TEST_RECEIVER_EMAIL - A different  email address you own that will receive the test emauk

  '''


import unittest
from unittest import TestCase

from tests.data_for_tests import *

from wholemail import BasicCodeVerificationEmailTemplate , GmailSender , BasicEmailVerificationEmailTemplate
from my_email_auth import GMAIL_EMAIL , GMAIL_APP_PASSWORD , TEST_RECIPIENT

test_sender = GmailSender(GMAIL_EMAIL , GMAIL_APP_PASSWORD )
template_style = 2
class Test_BasicEmailVerificationEmailTemplate(TestCase):

	def test_SimpleStyle(self):
		'''
		Just sending an email to see if everything works and looks as ot should
		'''

		sender = test_sender
		test_email_template = BasicEmailVerificationEmailTemplate(

					recipient_name = "JohnDoe" , 
					company_name = "WholemailApp",
					email_verification_link = "httpsverificationlink.com",
					company_address = "Mars",
					customer_support_link = "support",
					company_phone_number = "+123456",
					company_website_link = "wholemailwebsite",
					template_style = template_style,
					 )
		html  = test_email_template.content
		plain_message = test_email_template.plain_text_content
		subject = "Testing wholemail email verification email template"
		result = sender.SendHTMLEmail( TEST_RECIPIENT , subject , html , plain_message  )
		self.assertTrue(result)
		print("Email sent to {} for testing purposes".format(TEST_RECIPIENT))

class Test_BasicCodeVerificationEmailTemplate(TestCase):
	def test_SimpleStyle(self):
		'''
		Just sending an email to see if everything works and looks as ot should ,
		check your address to see the result
		'''

		sender = test_sender
		test_email_template = BasicCodeVerificationEmailTemplate(
					code = "12345",
					recipient_name = "JohnDoe" , 
					company_name = "WholemailApp",
					company_address = "Mars",
					customer_support_link = "support",
					company_phone_number = "+123456",
					company_website_link = "wholemailwebsite",
					template_style = template_style,
					)
		html  = test_email_template.content
		plain_message = test_email_template.plain_text_content
		subject = "Testing wholemail verification code email simple style"
		result = sender.SendHTMLEmail( TEST_RECIPIENT , subject , html , plain_message  )
		self.assertTrue(result)
		print("Email sent to {} for testing purposes".format(TEST_RECIPIENT))






import os

current_file = __file__
current_dir ,file_name = os.path.split(current_file)
html_boilerplate_dir = os.path.join(current_dir, "html_boilerplates")

# a hotfix function to create the html files
load_html_file = lambda file:os.path.join(html_boilerplate_dir , file)









import unittest 
from unittest import TestCase

from wholemail import EmailCodeVerifier

from tests.data_for_tests import test_code_verifiers

test_verifier = test_code_verifiers['auth']
test_email = "someemail@dom"
class Test_EmailCodeVerifier(TestCase):
	'''For testing the class EmailCodeVerifier

	'''

	def test_MakeNewCode(self):
		'''Test new code generations
		'''
		verifier = test_verifier
		verifier.ClearCodes()
		expected_code_length = verifier.length
		code = verifier.MakeNewCode(test_email)
		self.assertTrue(len(code) == expected_code_length)



	def test_ValidateCode(self):
		'''
		Test if  codes are validated correctly

		'''
		verifier = test_verifier
		verifier.ClearCodes()
		code = verifier.MakeNewCode(test_email)
		invalid_check_result= verifier.ValidateCode("falsecode123" , test_email)
		valid_check_result = verifier.ValidateCode(code , test_email)
		self.assertTrue(valid_check_result)
		self.assertFalse(invalid_check_result)

		# test when delete_if_valid is set to False
		verifier.ClearCodes()
		code = verifier.MakeNewCode(test_email)
		check_result = verifier.ValidateCode(code , test_email , delete_if_valid = False)
		storage_file_data = ''
		with open(verifier.code_generator.storage_file , "r") as f:
			storage_file_data = f.read()
		self.assertTrue(test_email in storage_file_data)
		self.assertTrue(code in storage_file_data)

		# test when delete_if_valid is set to True
		verifier.ClearCodes()
		code = verifier.MakeNewCode(test_email)
		check_result = verifier.ValidateCode(code , test_email , delete_if_valid = True)
		with open(verifier.code_generator.storage_file , "r") as f:
			storage_file_data = f.read()
		self.assertTrue(test_email not in storage_file_data)
		self.assertTrue(code not in storage_file_data)







import unittest 
from unittest import TestCase

from wholemail import EmailTemplateLoadingError
from wholemail import LoadTemplate


test_html_text = "<h3>{{title}}</h3><p>{{message}}</p>"
test_html_file = "test_data/test_html.html"
test_context = {
		"title":"testtitle",
		"message":"themessagefortesting"
}
expected_content_render = "<h3>{}</h3><p>{}</p>".format(test_context['title'] ,test_context['message'])

def PrepareTestFile():
	with open(test_html_file , "w" , encoding="utf-8") as f:
		f.write(test_html_text)


class TestTemplateLoading(TestCase):
	'''
	This TestCase class is for testing the template loading functionality of the wholemail package
	'''

	def test_HTMLContentLoading(self):
		'''Testing if the html content is loaded correctly
		'''

		PrepareTestFile()
		#Test if normal loading happens
		template = LoadTemplate(html_text = test_html_text , html_file = test_html_file, context = test_context )
		self.assertTrue(template.content == expected_content_render)
		self.assertTrue(template.html == test_html_text)

		# test if invalid file exception
		with self.assertRaises(EmailTemplateLoadingError):
			template = LoadTemplate(html_text = "" , html_file = "", context = test_context )

		# html_text is "" and html_file present
		template = LoadTemplate(html_text = "" , html_file = test_html_file, context = test_context )
		self.assertTrue(template.content == expected_content_render)